@@ -10,26 +10,86 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
 import warnings
 
-from openstack.block_storage import _base_proxy
 from openstack.block_storage.v2 import backup as _backup
 from openstack.block_storage.v2 import capabilities as _capabilities
 from openstack.block_storage.v2 import extension as _extension
 from openstack.block_storage.v2 import limits as _limits
 from openstack.block_storage.v2 import quota_class_set as _quota_class_set
 from openstack.block_storage.v2 import quota_set as _quota_set
+from openstack.block_storage.v2 import service as _service
 from openstack.block_storage.v2 import snapshot as _snapshot
 from openstack.block_storage.v2 import stats as _stats
+from openstack.block_storage.v2 import transfer as _transfer
 from openstack.block_storage.v2 import type as _type
 from openstack.block_storage.v2 import volume as _volume
+from openstack import exceptions
 from openstack.identity.v3 import project as _project
+from openstack import proxy
 from openstack import resource
+from openstack import utils
 from openstack import warnings as os_warnings
 
 
-class Proxy(_base_proxy.BaseBlockStorageProxy):
-    # ====== SNAPSHOTS ======
+class Proxy(proxy.Proxy):
+    # ========== Extensions ==========
+
+    def extensions(self):
+        """Return a generator of extensions
+
+        :returns: A generator of extension
+        :rtype: :class:`~openstack.block_storage.v2.extension.Extension`
+        """
+        return self._list(_extension.Extension)
+
+    # ========== Images ==========
+
+    # TODO(stephenfin): Convert to use resources/proxy rather than direct calls
+    def create_image(
+        self,
+        name,
+        volume,
+        allow_duplicates,
+        container_format,
+        disk_format,
+        wait,
+        timeout,
+    ):
+        if not disk_format:
+            disk_format = self._connection.config.config['image_format']
+        if not container_format:
+            # https://docs.openstack.org/image-guide/image-formats.html
+            container_format = 'bare'
+
+        if 'id' in volume:
+            volume_id = volume['id']
+        else:
+            volume_obj = self.get_volume(volume)
+            if not volume_obj:
+                raise exceptions.SDKException(
+                    f"Volume {volume} given to create_image could not be found"
+                )
+            volume_id = volume_obj['id']
+        data = self.post(
+            f'/volumes/{volume_id}/action',
+            json={
+                'os-volume_upload_image': {
+                    'force': allow_duplicates,
+                    'image_name': name,
+                    'container_format': container_format,
+                    'disk_format': disk_format,
+                }
+            },
+        )
+        response = self._connection._get_and_munchify(
+            'os-volume_upload_image', data
+        )
+        return self._connection.image._existing_image(id=response['image_id'])
+
+    # ========== Snapshots ==========
+
     def get_snapshot(self, snapshot):
         """Get a single snapshot
 
@@ -124,6 +184,18 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._create(_snapshot.Snapshot, **attrs)
 
+    def update_snapshot(self, snapshot, **attrs):
+        """Update a snapshot
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot` instance.
+        :param dict attrs: The attributes to update on the snapshot.
+
+        :returns: The updated snapshot
+        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
+        """
+        return self._update(_snapshot.Snapshot, snapshot, **attrs)
+
     def delete_snapshot(self, snapshot, ignore_missing=True):
         """Delete a snapshot
 
@@ -142,8 +214,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             _snapshot.Snapshot, snapshot, ignore_missing=ignore_missing
         )
 
-    # ====== SNAPSHOT ACTIONS ======
-    def reset_snapshot(self, snapshot, status):
+    # ========== Snapshot actions ==========
+
+    def reset_snapshot_status(self, snapshot, status):
         """Reset status of the snapshot
 
         :param snapshot: The value can be either the ID of a backup or a
@@ -153,9 +226,42 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
-        snapshot.reset(self, status)
+        snapshot.reset_status(self, status)
+
+    def reset_snapshot(self, snapshot, status):
+        warnings.warn(
+            "reset_snapshot is a deprecated alias for reset_snapshot_status "
+            "and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.reset_snapshot_status(snapshot, status)
+
+    def manage_snapshot(self, **attrs):
+        """Creates a snapshot by using existing storage rather than
+        allocating new storage.
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v2.snapshot.Snapshot`,
+            comprised of the properties on the Snapshot class.
+
+        :returns: The results of snapshot creation
+        :rtype: :class:`~openstack.block_storage.v2.snapshot.Snapshot`
+        """
+        return _snapshot.Snapshot.manage(self, **attrs)
+
+    def unmanage_snapshot(self, snapshot):
+        """Unmanage a snapshot from block storage provisioning.
+
+        :param snapshot: Either the ID of a snapshot or a
+            :class:`~openstack.block_storage.v2.snapshot.Snapshot`.
+
+        :returns: None
+        """
+        snapshot_obj = self._get_resource(_snapshot.Snapshot, snapshot)
+        snapshot_obj.unmanage(self)
+
+    # ========== Types ==========
 
-    # ====== TYPES ======
     def get_type(self, type):
         """Get a single type
 
@@ -260,7 +366,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         res = self._get_resource(_type.Type, type)
         return res.remove_private_access(self, project_id)
 
-    # ====== VOLUMES ======
+    # ========== Volumes ==========
+
     def get_volume(self, volume):
         """Get a single volume
 
@@ -347,7 +454,9 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._create(_volume.Volume, **attrs)
 
-    def delete_volume(self, volume, ignore_missing=True, force=False):
+    def delete_volume(
+        self, volume, ignore_missing=True, *, force=False, cascade=False
+    ):
         """Delete a volume
 
         :param volume: The value can be either the ID of a volume or a
@@ -358,16 +467,24 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             exception will be set when attempting to delete a nonexistent
             volume.
         :param bool force: Whether to try forcing volume deletion.
+        :param bool cascade: Whether to remove any snapshots along with the
+            volume.
 
         :returns: ``None``
         """
-        if not force:
-            self._delete(_volume.Volume, volume, ignore_missing=ignore_missing)
-        else:
-            volume = self._get_resource(_volume.Volume, volume)
-            volume.force_delete(self)
+        volume = self._get_resource(_volume.Volume, volume)
+        try:
+            if not force:
+                volume.delete(self, params={'cascade': cascade})
+            else:
+                volume.force_delete(self)
+        except exceptions.NotFoundException:
+            if ignore_missing:
+                return None
+            raise
+
+    # ========== Volume actions ==========
 
-    # ====== VOLUME ACTIONS ======
     def extend_volume(self, volume, size):
         """Extend a volume
 
@@ -548,7 +665,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         volume = self._get_resource(_volume.Volume, volume)
         volume.complete_migration(self, new_volume, error)
 
-    # ====== BACKEND POOLS ======
+    # ========== Backend pools ==========
+
     def backend_pools(self, **query):
         """Returns a generator of cinder Back-end storage pools
 
@@ -559,7 +677,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._list(_stats.Pools, **query)
 
-    # ====== BACKUPS ======
+    # ========== Backups ==========
+
     def backups(self, details=True, **query):
         """Retrieve a generator of backups
 
@@ -654,7 +773,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             backup = self._get_resource(_backup.Backup, backup)
             backup.force_delete(self)
 
-    # ====== BACKUP ACTIONS ======
+    # ========== Backup actions ==========
+
     def restore_backup(self, backup, volume_id, name):
         """Restore a Backup to volume
 
@@ -669,7 +789,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         backup = self._get_resource(_backup.Backup, backup)
         return backup.restore(self, volume_id=volume_id, name=name)
 
-    def reset_backup(self, backup, status):
+    def reset_backup_status(self, backup, status):
         """Reset status of the backup
 
         :param backup: The value can be either the ID of a backup or a
@@ -679,9 +799,18 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         :returns: None
         """
         backup = self._get_resource(_backup.Backup, backup)
-        backup.reset(self, status)
+        backup.reset_status(self, status)
+
+    def reset_backup(self, backup, status):
+        warnings.warn(
+            "reset_backup is a deprecated alias for reset_backup_status "
+            "and will be removed in a future release.",
+            os_warnings.RemovedInSDK60Warning,
+        )
+        return self.reset_backup_status(backup, status)
+
+    # ========== Limits ==========
 
-    # ====== LIMITS ======
     def get_limits(self, project=None):
         """Retrieves limits
 
@@ -693,12 +822,16 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             :class:`~openstack.block_storage.v2.limits.RateLimit`
         :rtype: :class:`~openstack.block_storage.v2.limits.Limits`
         """
-        params = {}
         if project:
-            params['project_id'] = resource.Resource._get_id(project)
-        return self._get(_limits.Limits, requires_id=False, **params)
+            return self._get(
+                _limits.Limits,
+                requires_id=False,
+                project_id=resource.Resource._get_id(project),
+            )
+        return self._get(_limits.Limits, requires_id=False)
+
+    # ========== Capabilities ==========
 
-    # ====== CAPABILITIES ======
     def get_capabilities(self, host):
         """Get a backend's capabilites
 
@@ -711,7 +844,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         """
         return self._get(_capabilities.Capabilities, host)
 
-    # ====== QUOTA CLASS SETS ======
+    # ========== Quota class sets ==========
+
     def get_quota_class_set(self, quota_class_set='default'):
         """Get a single quota class set
 
@@ -748,7 +882,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             _quota_class_set.QuotaClassSet, quota_class_set, **attrs
         )
 
-    # ====== QUOTA SETS ======
+    # ========== Quota sets ==========
+
     def get_quota_set(self, project, usage=False, **query):
         """Show QuotaSet information for the project
 
@@ -815,7 +950,7 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             by ``quota_set``.
 
         :returns: The updated QuotaSet
-        :rtype: :class:`~openstack.block_storage.v3.quota_set.QuotaSet`
+        :rtype: :class:`~openstack.block_storage.v2.quota_set.QuotaSet`
         """
         if 'project_id' in attrs or isinstance(project, _quota_set.QuotaSet):
             warnings.warn(
@@ -841,7 +976,159 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
             attrs['project_id'] = project.id
             return self._update(_quota_set.QuotaSet, None, **attrs)
 
-    # ====== VOLUME METADATA ======
+    # ========== Services ==========
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: ty.Literal[True] = True,
+        **query: ty.Any,
+    ) -> _service.Service | None: ...
+
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: ty.Literal[False],
+        **query: ty.Any,
+    ) -> _service.Service: ...
+
+    # excuse the duplication here: it's mypy's fault
+    # https://github.com/python/mypy/issues/14764
+    @ty.overload
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: bool,
+        **query: ty.Any,
+    ) -> _service.Service | None: ...
+
+    def find_service(
+        self,
+        name_or_id: str,
+        ignore_missing: bool = True,
+        **query: ty.Any,
+    ) -> _service.Service | None:
+        """Find a single service
+
+        :param name_or_id: The name or ID of a service
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
+            the resource does not exist.
+            When set to ``True``, None will be returned when attempting to find
+            a nonexistent resource.
+        :param dict query: Additional attributes like 'host'
+
+        :returns: One: class:`~openstack.block_storage.v2.service.Service` or None
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
+            resources are found.
+        """
+        return self._find(
+            _service.Service,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            **query,
+        )
+
+    def services(
+        self,
+        **query: ty.Any,
+    ) -> ty.Generator[_service.Service, None, None]:
+        """Return a generator of service
+
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
+        :returns: A generator of Service objects
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        return self._list(_service.Service, **query)
+
+    def enable_service(
+        self,
+        service: str | _service.Service,
+    ) -> _service.Service:
+        """Enable a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v2.service.Service` instance.
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        service_obj = self._get_resource(_service.Service, service)
+        return service_obj.enable(self)
+
+    def disable_service(
+        self,
+        service: str | _service.Service,
+        *,
+        reason: str | None = None,
+    ) -> _service.Service:
+        """Disable a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v2.service.Service` instance
+        :param str reason: The reason to disable a service
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        service_obj = self._get_resource(_service.Service, service)
+        return service_obj.disable(self, reason=reason)
+
+    def thaw_service(
+        self,
+        service: str | _service.Service,
+    ) -> _service.Service:
+        """Thaw a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v2.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        service_obj = self._get_resource(_service.Service, service)
+        return service_obj.thaw(self)
+
+    def freeze_service(
+        self,
+        service: str | _service.Service,
+    ) -> _service.Service:
+        """Freeze a service
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v2.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        service_obj = self._get_resource(_service.Service, service)
+        return service_obj.freeze(self)
+
+    def failover_service(
+        self,
+        service: str | _service.Service,
+        *,
+        backend_id: str | None = None,
+    ) -> _service.Service:
+        """Failover a service
+
+        Only applies to replicating cinder-volume services.
+
+        :param service: Either the ID of a service or a
+            :class:`~openstack.block_storage.v2.service.Service` instance
+
+        :returns: Updated service instance
+        :rtype: class: `~openstack.block_storage.v2.service.Service`
+        """
+        service_obj = self._get_resource(_service.Service, service)
+        return service_obj.failover(self, backend_id=backend_id)
+
+    # ========== Volume metadata ==========
+
     def get_volume_metadata(self, volume):
         """Return a dictionary of metadata for a volume
 
@@ -888,7 +1175,8 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         else:
             volume.delete_metadata(self)
 
-    # ====== SNAPSHOT METADATA ======
+    # ========== Snapshot metadata ==========
+
     def get_snapshot_metadata(self, snapshot):
         """Return a dictionary of metadata for a snapshot
 
@@ -937,79 +1225,169 @@ class Proxy(_base_proxy.BaseBlockStorageProxy):
         else:
             snapshot.delete_metadata(self)
 
-    # ====== EXTENSIONS ======
-    def extensions(self):
-        """Return a generator of extensions
+    # ========== Transfers ==========
 
-        :returns: A generator of extension
-        :rtype: :class:`~openstack.block_storage.v2.extension.Extension`
+    def create_transfer(self, **attrs):
+        """Create a new Transfer record
+
+        :param volume_id: The value is ID of the volume.
+        :param name: The value is name of the transfer
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.block_storage.v2.transfer.Transfer`
+            comprised of the properties on the Transfer class.
+        :returns: The results of Transfer creation
+        :rtype: :class:`~openstack.block_storage.v2.transfer.Transfer`
         """
-        return self._list(_extension.Extension)
+        return self._create(_transfer.Transfer, **attrs)
 
-    # ====== UTILS ======
-    def wait_for_status(
-        self,
-        res,
-        status='available',
-        failures=None,
-        interval=2,
-        wait=120,
-        callback=None,
-    ):
-        """Wait for a resource to be in a particular status.
+    def delete_transfer(self, transfer, ignore_missing=True):
+        """Delete a volume transfer
 
-        :param res: The resource to wait on to reach the specified status.
-            The resource must have a ``status`` attribute.
-        :type resource: A :class:`~openstack.resource.Resource` object.
-        :param status: Desired status.
-        :param failures: Statuses that would be interpreted as failures.
-        :type failures: :py:class:`list`
-        :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
-        :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
-        :param callback: A callback function. This will be called with a single
-            value, progress.
+        :param transfer: The value can be either the ID of a transfer or a
+            :class:`~openstack.block_storage.v2.transfer.Transfer`` instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be
+            raised when the transfer does not exist.
+            When set to ``True``, no exception will be set when
+            attempting to delete a nonexistent transfer.
 
-        :returns: The resource is returned on success.
-        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
-            to the desired status failed to occur in specified seconds.
-        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
-            has transited to one of the failure statuses.
-        :raises: :class:`~AttributeError` if the resource does not have a
-            ``status`` attribute.
+        :returns: ``None``
         """
-        failures = ['error'] if failures is None else failures
-        return resource.wait_for_status(
-            self,
-            res,
-            status,
-            failures,
-            interval,
-            wait,
-            callback=callback,
+        self._delete(
+            _transfer.Transfer,
+            transfer,
+            ignore_missing=ignore_missing,
         )
 
-    def wait_for_delete(self, res, interval=2, wait=120, callback=None):
+    def find_transfer(self, name_or_id, ignore_missing=True):
+        """Find a single transfer
+
+        :param name_or_id: The name or ID a transfer
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the volume transfer does not exist.
+
+        :returns: One :class:`~openstack.block_storage.v2.transfer.Transfer`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        :raises: :class:`~openstack.exceptions.DuplicateResource` when multiple
+            resources are found.
+        """
+        return self._find(
+            _transfer.Transfer,
+            name_or_id,
+            ignore_missing=ignore_missing,
+        )
+
+    def get_transfer(self, transfer):
+        """Get a single transfer
+
+        :param transfer: The value can be the ID of a transfer or a
+            :class:`~openstack.block_storage.v2.transfer.Transfer`
+            instance.
+
+        :returns: One :class:`~openstack.block_storage.v2.transfer.Transfer`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
+        """
+        return self._get(_transfer.Transfer, transfer)
+
+    def transfers(self, *, details=True, all_projects=False, **query):
+        """Retrieve a generator of transfers
+
+        :param bool details: When set to ``False`` no extended attributes
+            will be returned. The default, ``True``, will cause objects with
+            additional attributes to be returned.
+        :param bool all_projects: When set to ``True``, list transfers from
+            all projects. Admin-only by default.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the transfers being returned.
+
+        :returns: A generator of transfer objects.
+        """
+        if all_projects:
+            query['all_projects'] = True
+        base_path = '/volume-transfers'
+        if details:
+            base_path = utils.urljoin(base_path, 'detail')
+        return self._list(_transfer.Transfer, base_path=base_path, **query)
+
+    def accept_transfer(self, transfer_id, auth_key):
+        """Accept a Transfer
+
+        :param transfer_id: The value can be the ID of a transfer or a
+            :class:`~openstack.block_storage.v2.transfer.Transfer`
+            instance.
+        :param auth_key: The key to authenticate volume transfer.
+
+        :returns: The results of Transfer creation
+        :rtype: :class:`~openstack.block_storage.v2.transfer.Transfer`
+        """
+        transfer = self._get_resource(_transfer.Transfer, transfer_id)
+        return transfer.accept(self, auth_key=auth_key)
+
+    # ========== Utilities ==========
+
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str = 'available',
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
+        attribute: str = 'status',
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        if failures is None:
+            failures = ['error']
+
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
         """Wait for a resource to be deleted.
 
         :param res: The resource to wait on to be deleted.
-        :type resource: A :class:`~openstack.resource.Resource` object.
         :param interval: Number of seconds to wait before to consecutive
-            checks. Default to 2.
+            checks.
         :param wait: Maximum number of seconds to wait before the change.
-            Default to 120.
         :param callback: A callback function. This will be called with a single
-            value, progress.
+            value, progress, which is a percentage value from 0-100.
 
         :returns: The resource is returned on success.
         :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
             to delete failed to occur in the specified seconds.
         """
-        return resource.wait_for_delete(
-            self,
-            res,
-            interval,
-            wait,
-            callback=callback,
-        )
+        return resource.wait_for_delete(self, res, interval, wait, callback)
