@@ -9,6 +9,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
 from openstack.common import metadata
 from openstack.common import tag
 from openstack.compute.v2 import flavor
@@ -16,6 +19,7 @@ from openstack.compute.v2 import volume_attachment
 from openstack import exceptions
 from openstack.image.v2 import image
 from openstack import resource
+from openstack import types
 from openstack import utils
 
 
@@ -23,6 +27,7 @@ CONSOLE_TYPE_ACTION_MAPPING = {
     'novnc': 'os-getVNCConsole',
     'xvpvnc': 'os-getVNCConsole',
     'spice-html5': 'os-getSPICEConsole',
+    'spice-direct': 'os-getSPICEConsole',
     'rdp-html5': 'os-getRDPConsole',
     'serial': 'os-getSerialConsole',
 }
@@ -40,11 +45,6 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     allow_delete = True
     allow_list = True
 
-    # Sentinel used to differentiate API called without parameter or None
-    # Ex unshelve API can be called without an availability_zone or with
-    # availability_zone = None to unpin the az.
-    _sentinel = object()
-
     _query_mapping = resource.QueryParameters(
         "auto_disk_config",
         "availability_zone",
@@ -57,6 +57,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         "key_name",
         "launch_index",
         "launched_at",
+        "locked",
         "locked_by",
         "name",
         "node",
@@ -73,6 +74,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         "vm_state",
         "sort_key",
         "sort_dir",
+        "pinned_availability_zone",
         access_ipv4="access_ip_v4",
         access_ipv6="access_ip_v6",
         has_config_drive="config_drive",
@@ -88,7 +90,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         **tag.TagMixin._tag_query_parameters,
     )
 
-    _max_microversion = '2.91'
+    _max_microversion = '2.100'
 
     #: A list of dictionaries holding links relevant to this server.
     links = resource.Body('links')
@@ -143,6 +145,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: A fault object. Only available when the server status
     #: is ERROR or DELETED and a fault occurred.
     fault = resource.Body('fault')
+    #: The host to boot the server on.
+    host = resource.Body('host')
     #: The host status.
     host_status = resource.Body('host_status')
     #: The hostname set on the instance when it is booted.
@@ -165,7 +169,6 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: routable ipv6 addresses, and to private_ipv4 if the Connection was
     #: created with private=True. Otherwise it will be set to public_ipv4.
     interface_ip = resource.Computed('interface_ip', default='')
-
     # The locked status of the server
     is_locked = resource.Body('locked', type=bool)
     #: The UUID of the kernel image when using an AMI. Will be null if not.
@@ -179,6 +182,8 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     launch_index = resource.Body('OS-EXT-SRV-ATTR:launch_index', type=int)
     #: The timestamp when the server was launched.
     launched_at = resource.Body('OS-SRV-USG:launched_at')
+    #: The reason the server was locked, if any.
+    locked_reason = resource.Body('locked_reason')
     #: The maximum number of servers to create.
     max_count = resource.Body('max_count')
     #: The minimum number of servers to create.
@@ -188,6 +193,14 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
     #: networks parameter, the server attaches to the only network
     #: created for the current tenant.
     networks = resource.Body('networks')
+    #: Personality files. This should be a list of dicts with each dict
+    #: containing a file name ('name') and a base64-encoded file contents
+    #: ('contents')
+    personality = resource.Body('personality', type=list)
+    #: The availability zone requested during server creation OR pinned
+    #: availability zone, which is configured using default_schedule_zone
+    #: config option.
+    pinned_availability_zone = resource.Body('pinned_availability_zone')
     #: The power state of this server.
     power_state = resource.Body('OS-EXT-STS:power_state')
     #: While the server is building, this value represents the percentage
@@ -256,13 +269,18 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         requires_id=True,
         prepend_key=True,
+        patch=False,
         base_path=None,
+        params=None,
         **kwargs,
     ):
-        request = super(Server, self)._prepare_request(
+        request = super()._prepare_request(
             requires_id=requires_id,
             prepend_key=prepend_key,
+            patch=patch,
             base_path=base_path,
+            params=params,
+            **kwargs,
         )
 
         server_body = request.body[self.resource_key]
@@ -303,7 +321,7 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         # these aren't all necessary "commit" actions (i.e. updates) but it's
         # good enough...
         if microversion is None:
-            microversion = self._get_microversion(session, action='commit')
+            microversion = self._get_microversion(session)
 
         response = session.post(
             url,
@@ -314,28 +332,64 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         exceptions.raise_from_response(response)
         return response
 
-    def change_password(self, session, new_password):
-        """Change the administrator password to the given password."""
-        body = {'changePassword': {'adminPass': new_password}}
-        self._action(session, body)
+    def change_password(self, session, password, *, microversion=None):
+        """Change the administrator password to the given password.
 
-    def get_password(self, session):
-        """Get the encrypted administrator password."""
+        :param session: The session to use for making this request.
+        :param password: The new password.
+        :returns: None
+        """
+        body = {'changePassword': {'adminPass': password}}
+        self._action(session, body, microversion=microversion)
+
+    def get_password(self, session, *, microversion=None):
+        """Get the encrypted administrator password.
+
+        :param session: The session to use for making this request.
+        :returns: The encrypted administrator password.
+        """
         url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session)
 
-        response = session.get(url)
+        response = session.get(url, microversion=microversion)
         exceptions.raise_from_response(response)
 
         data = response.json()
         return data.get('password')
 
+    def clear_password(self, session, *, microversion=None):
+        """Clear the administrator password.
+
+        This removes the password from the database. It does not actually
+        change the server password.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
+        url = utils.urljoin(Server.base_path, self.id, 'os-server-password')
+        if microversion is None:
+            microversion = self._get_microversion(session)
+
+        response = session.delete(url, microversion=microversion)
+        exceptions.raise_from_response(response)
+
     def reboot(self, session, reboot_type):
-        """Reboot server where reboot_type might be 'SOFT' or 'HARD'."""
+        """Reboot server where reboot_type might be 'SOFT' or 'HARD'.
+
+        :param session: The session to use for making this request.
+        :param reboot_type: The type of reboot. One of: ``SOFT``, ``HARD``.
+        :returns: None
+        """
         body = {'reboot': {'type': reboot_type}}
         self._action(session, body)
 
     def force_delete(self, session):
-        """Force delete a server."""
+        """Force delete the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {'forceDelete': None}
         self._action(session, body)
 
@@ -343,33 +397,67 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self,
         session,
         image,
-        name=None,
-        admin_password=None,
-        preserve_ephemeral=None,
-        access_ipv4=None,
-        access_ipv6=None,
-        metadata=None,
-        user_data=None,
-        key_name=None,
+        name=types.UNSET,
+        admin_password=types.UNSET,
+        preserve_ephemeral=types.UNSET,
+        access_ipv4=types.UNSET,
+        access_ipv6=types.UNSET,
+        metadata=types.UNSET,
+        user_data=types.UNSET,
+        key_name=types.UNSET,
+        description=types.UNSET,
+        trusted_image_certificates=types.UNSET,
+        hostname=types.UNSET,
     ):
-        """Rebuild the server with the given arguments."""
+        """Rebuild the server with the given arguments.
+
+        :param session: The session to use for making this request.
+        :param image: The image to rebuild to. Either an ID or a
+            :class:`~openstack.image.v1.image.Image` instance.
+        :param name: A name to set on the rebuilt server. (Optional)
+        :param admin_password: An admin password to set on the rebuilt server.
+            (Optional)
+        :param preserve_ephemeral: Whether to preserve the ephemeral drive
+            during the rebuild. (Optional)
+        :param access_ipv4: An IPv4 address that will be used to access the
+            rebuilt server. (Optional)
+        :param access_ipv6: An IPv6 address that will be used to access the
+            rebuilt server. (Optional)
+        :param metadata: Metadata to set on the updated server. (Optional)
+        :param user_data: User data to set on the updated server. (Optional)
+        :param key_name: A key name to set on the updated server. (Optional)
+        :param description: The description to set on the updated server.
+            (Optional) (Requires API microversion 2.19)
+        :param trusted_image_certificates: The trusted image certificates to
+            set on the updated server. (Optional) (Requires API microversion
+            2.78)
+        :param hostname: The hostname to set on the updated server. (Optional)
+            (Requires API microversion 2.90)
+        :returns: The updated server.
+        """
         action = {'imageRef': resource.Resource._get_id(image)}
-        if preserve_ephemeral is not None:
+        if preserve_ephemeral is not types.UNSET:
             action['preserve_ephemeral'] = preserve_ephemeral
-        if name is not None:
+        if name is not types.UNSET:
             action['name'] = name
-        if admin_password is not None:
+        if admin_password is not types.UNSET:
             action['adminPass'] = admin_password
-        if access_ipv4 is not None:
+        if access_ipv4 is not types.UNSET:
             action['accessIPv4'] = access_ipv4
-        if access_ipv6 is not None:
+        if access_ipv6 is not types.UNSET:
             action['accessIPv6'] = access_ipv6
-        if metadata is not None:
+        if metadata is not types.UNSET:
             action['metadata'] = metadata
-        if user_data is not None:
+        if user_data is not types.UNSET:
             action['user_data'] = user_data
-        if key_name is not None:
+        if key_name is not types.UNSET:
             action['key_name'] = key_name
+        if description is not types.UNSET:
+            action['description'] = description
+        if trusted_image_certificates is not types.UNSET:
+            action['trusted_image_certificates'] = trusted_image_certificates
+        if hostname is not types.UNSET:
+            action['hostname'] = hostname
 
         body = {'rebuild': action}
         response = self._action(session, body)
@@ -377,22 +465,41 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         return self
 
     def resize(self, session, flavor):
-        """Resize server to flavor reference."""
+        """Resize server to flavor reference.
+
+        :param session: The session to use for making this request.
+        :param flavor: The server to resize to.
+        :returns: None
+        """
         body = {'resize': {'flavorRef': flavor}}
         self._action(session, body)
 
     def confirm_resize(self, session):
-        """Confirm the resize of the server."""
+        """Confirm the resize of the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {'confirmResize': None}
         self._action(session, body)
 
     def revert_resize(self, session):
-        """Revert the resize of the server."""
+        """Revert the resize of the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {'revertResize': None}
         self._action(session, body)
 
     def create_image(self, session, name, metadata=None):
-        """Create image from server."""
+        """Create image from server.
+
+        :param session: The session to use for making this request.
+        :param name: The name to use for the created image.
+        :param metadata: Metadata to set on the created image. (Optional)
+        :returns: None
+        """
         action = {'name': name}
         if metadata is not None:
             action['metadata'] = metadata
@@ -418,50 +525,113 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             microversion = '2.45'
         response = self._action(session, body, microversion)
 
-        body = None
         try:
-            # There might be body, might be not
-            body = response.json()
+            # There might be a body, there might not be
+            response_body = response.json()
         except Exception:
-            pass
-        if body and 'image_id' in body:
-            image_id = body['image_id']
+            response_body = None
+        if response_body and 'image_id' in response_body:
+            image_id = response_body['image_id']
         else:
             image_id = response.headers['Location'].rsplit('/', 1)[1]
 
         return image_id
 
     def add_security_group(self, session, security_group_name):
+        """Add a security group to the server.
+
+        :param session: The session to use for making this request.
+        :param security_group_name: The security group to add to the server.
+        :returns: None
+        """
         body = {"addSecurityGroup": {"name": security_group_name}}
         self._action(session, body)
 
     def remove_security_group(self, session, security_group_name):
+        """Remove a security group from the server.
+
+        :param session: The session to use for making this request.
+        :param security_group_name: The security group to remove from the
+            server.
+        :returns: None
+        """
         body = {"removeSecurityGroup": {"name": security_group_name}}
         self._action(session, body)
 
     def reset_state(self, session, state):
+        """Reset server state.
+
+        This is admin-only by default.
+
+        :param session: The session to use for making this request.
+        :param state: The state to set on the server.
+        :returns: None
+        """
         body = {"os-resetState": {"state": state}}
         self._action(session, body)
 
     def add_fixed_ip(self, session, network_id):
+        """Add a fixed IP to the server.
+
+        This is effectively an alias for adding a network.
+
+        :param session: The session to use for making this request.
+        :param network_id: The network to connect the server to.
+        :returns: None
+        """
         body = {"addFixedIp": {"networkId": network_id}}
         self._action(session, body)
 
     def remove_fixed_ip(self, session, address):
+        """Remove a fixed IP from the server.
+
+        This is effectively an alias from removing a port from the server.
+
+        :param session: The session to use for making this request.
+        :param network_id: The address to remove from the server.
+        :returns: None
+        """
         body = {"removeFixedIp": {"address": address}}
         self._action(session, body)
 
     def add_floating_ip(self, session, address, fixed_address=None):
+        """Add a floating IP to the server.
+
+        :param session: The session to use for making this request.
+        :param address: The floating IP address to associate with the server.
+        :param fixed_address: A fixed IP address with which to associated the
+            floating IP. (Optional)
+        :returns: None
+        """
         body = {"addFloatingIp": {"address": address}}
         if fixed_address is not None:
             body['addFloatingIp']['fixed_address'] = fixed_address
         self._action(session, body)
 
     def remove_floating_ip(self, session, address):
+        """Remove a floating IP from the server.
+
+        :param session: The session to use for making this request.
+        :param address: The floating IP address to disassociate from the
+            server.
+        :returns: None
+        """
         body = {"removeFloatingIp": {"address": address}}
         self._action(session, body)
 
     def backup(self, session, name, backup_type, rotation):
+        """Create a backup of the server.
+
+        :param session: The session to use for making this request.
+        :param name: The name to use for the backup image.
+        :param backup_type: The type of backup. The value and meaning of this
+            atribute is user-defined and can be used to separate backups of
+            different types. For example, this could be used to distinguish
+            between ``daily`` and ``weekly`` backups.
+        :param rotation: The number of backups to retain. All images older than
+            the rotation'th image will be deleted.
+        :returns: None
+        """
         body = {
             "createBackup": {
                 "name": name,
@@ -472,23 +642,49 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self._action(session, body)
 
     def pause(self, session):
+        """Pause the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"pause": None}
         self._action(session, body)
 
     def unpause(self, session):
+        """Unpause the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"unpause": None}
         self._action(session, body)
 
     def suspend(self, session):
+        """Suspend the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"suspend": None}
         self._action(session, body)
 
     def resume(self, session):
+        """Resume the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"resume": None}
         self._action(session, body)
 
     def lock(self, session, locked_reason=None):
-        body = {"lock": None}
+        """Lock the server.
+
+        :param session: The session to use for making this request.
+        :param locked_reason: The reason for locking the server.
+        :returns: None
+        """
+        body: dict[str, ty.Any] = {"lock": None}
         if locked_reason is not None:
             body["lock"] = {
                 "locked_reason": locked_reason,
@@ -496,11 +692,27 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self._action(session, body)
 
     def unlock(self, session):
+        """Unlock the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"unlock": None}
         self._action(session, body)
 
     def rescue(self, session, admin_pass=None, image_ref=None):
-        body = {"rescue": {}}
+        """Rescue the server.
+
+        This is admin-only by default.
+
+        :param session: The session to use for making this request.
+        :param admin_pass: A new admin password to set on the rescued server.
+            (Optional)
+        :param image_ref: The image to use when rescuing the server. If not
+            provided, the server will use the existing image. (Optional)
+        :returns: None
+        """
+        body: dict[str, ty.Any] = {"rescue": {}}
         if admin_pass is not None:
             body["rescue"]["adminPass"] = admin_pass
         if image_ref is not None:
@@ -508,43 +720,98 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         self._action(session, body)
 
     def unrescue(self, session):
+        """Unrescue the server.
+
+        This is admin-only by default.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"unrescue": None}
         self._action(session, body)
 
-    def evacuate(self, session, host=None, admin_pass=None, force=None):
-        body = {"evacuate": {}}
+    def evacuate(
+        self,
+        session,
+        host=None,
+        admin_pass=None,
+        force=None,
+        on_shared_storage=None,
+    ):
+        """Evacuate the server.
+
+        :param session: The session to use for making this request.
+        :param host: The host to evacuate the instance to. (Optional)
+        :param admin_pass: The admin password to set on the evacuated instance.
+            (Optional)
+        :param force: Whether to force evacuation.
+        :param on_shared_storage: Whether the host is using shared storage.
+            (Optional) (Only supported before microversion 2.14)
+        :returns: None
+        """
+        body: dict[str, ty.Any] = {"evacuate": {}}
         if host is not None:
             body["evacuate"]["host"] = host
         if admin_pass is not None:
             body["evacuate"]["adminPass"] = admin_pass
         if force is not None:
             body["evacuate"]["force"] = force
+        if on_shared_storage is not None:
+            body["evacuate"]["onSharedStorage"] = on_shared_storage
         self._action(session, body)
 
     def start(self, session):
+        """Start the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"os-start": None}
         self._action(session, body)
 
     def stop(self, session):
+        """Stop the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"os-stop": None}
         self._action(session, body)
 
     def restore(self, session):
+        """Restore the server.
+
+        This is only supported if the server is soft-deleted. This is
+        cloud-specific.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"restore": None}
         self._action(session, body)
 
     def shelve(self, session):
+        """Shelve the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"shelve": None}
         self._action(session, body)
 
     def shelve_offload(self, session):
+        """Shelve-offload the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"shelveOffload": None}
         self._action(session, body)
 
-    def unshelve(self, session, availability_zone=_sentinel, host=None):
-        """
-        Unshelve -- Unshelve the server.
+    def unshelve(self, session, availability_zone=types.UNSET, host=None):
+        """Unshelve the server.
 
+        :param session: The session to use for making this request.
         :param availability_zone: If specified the instance will be unshelved
             to the availability_zone.
             If None is passed the instance defined availability_zone is unpin
@@ -563,25 +830,60 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         body = {'unshelve': data or None}
         self._action(session, body)
 
-    def migrate(self, session):
-        body = {"migrate": None}
+    def migrate(self, session, *, host=None):
+        """Migrate the server.
+
+        :param session: The session to use for making this request.
+        :param host: The host to migrate the server to. (Optional)
+        :returns: None
+        """
+        if host and not utils.supports_microversion(session, '2.56'):
+            raise ValueError(
+                "The 'host' option is only supported on microversion 2.56 or "
+                "greater."
+            )
+
+        body: dict[str, ty.Any] = {"migrate": None}
+        if host:
+            body["migrate"] = {"host": host}
         self._action(session, body)
 
     def trigger_crash_dump(self, session):
+        """Trigger a crash dump for the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
         body = {"trigger_crash_dump": None}
         self._action(session, body)
 
     def get_console_output(self, session, length=None):
-        body = {"os-getConsoleOutput": {}}
+        """Get console output for the server.
+
+        :param session: The session to use for making this request.
+        :param length: The max length of the console output to return.
+            (Optional)
+        :returns: None
+        """
+        body: dict[str, ty.Any] = {"os-getConsoleOutput": {}}
         if length is not None:
             body["os-getConsoleOutput"]["length"] = length
         resp = self._action(session, body)
         return resp.json()
 
     def get_console_url(self, session, console_type):
+        """Get the console URL for the server.
+
+        :param session: The session to use for making this request.
+        :param console_type: The type of console to return. This is
+            cloud-specific. One of: ``novnc``, ``xvpvnc``, ``spice-html5``,
+            ``spice-direct`` (after Nova microversion 2.99), ``rdp-html5``,
+            or ``serial``.
+        :returns: None
+        """
         action = CONSOLE_TYPE_ACTION_MAPPING.get(console_type)
         if not action:
-            raise ValueError("Unsupported console type %s" % console_type)
+            raise ValueError(f"Unsupported console type {console_type}")
         body = {action: {'type': console_type}}
         resp = self._action(session, body)
         return resp.json().get('console')
@@ -594,6 +896,17 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         block_migration,
         disk_over_commit=False,
     ):
+        """Live migrate the server.
+
+        :param session: The session to use for making this request.
+        :param host: The host to live migrate the server to. (Optional)
+        :param force: Whether to force the migration. (Optional)
+        :param block_migration: Whether to do block migration. One of:
+            ``True``, ``False``, ``'auto'``. (Optional)
+        :param disk_over_commit: Whether to allow disk over-commit on the
+            destination host. (Optional)
+        :returns: None
+        """
         if utils.supports_microversion(session, '2.30'):
             return self._live_migrate_30(
                 session,
@@ -643,12 +956,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
@@ -665,13 +978,13 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         disk_over_commit,
     ):
         microversion = None
-        body = {
+        body: dict[str, ty.Any] = {
             'host': None,
         }
         if block_migration == 'auto':
             raise ValueError(
-                "Live migration on this cloud does not support 'auto' as"
-                " a parameter to block_migration, but only True and False."
+                "Live migration on this cloud does not support 'auto' as "
+                "a parameter to block_migration, but only True and False."
             )
         body['block_migration'] = block_migration or False
         body['disk_over_commit'] = disk_over_commit or False
@@ -679,12 +992,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             body['host'] = host
             if not force:
                 raise ValueError(
-                    "Live migration on this cloud implies 'force'"
-                    " if the 'host' option has been given and it is not"
-                    " possible to disable. It is recommended to not use 'host'"
-                    " at all on this cloud as it is inherently unsafe, but if"
-                    " it is unavoidable, please supply 'force=True' so that it"
-                    " is clear you understand the risks."
+                    "Live migration on this cloud implies 'force' "
+                    "if the 'host' option has been given and it is not "
+                    "possible to disable. It is recommended to not use 'host' "
+                    "at all on this cloud as it is inherently unsafe, but if "
+                    "it is unavoidable, please supply 'force=True' so that it "
+                    "is clear you understand the risks."
                 )
         self._action(
             session,
@@ -693,7 +1006,12 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
         )
 
     def fetch_topology(self, session):
-        utils.require_microversion(session, 2.78)
+        """Fetch the topology information for the server.
+
+        :param session: The session to use for making this request.
+        :returns: None
+        """
+        utils.require_microversion(session, '2.78')
 
         url = utils.urljoin(Server.base_path, self.id, 'topology')
 
@@ -707,10 +1025,10 @@ class Server(resource.Resource, metadata.MetadataMixin, tag.TagMixin):
             pass
 
     def fetch_security_groups(self, session):
-        """Fetch security groups of a server.
+        """Fetch security groups of the server.
 
+        :param session: The session to use for making this request.
         :returns: Updated Server instance.
-
         """
         url = utils.urljoin(Server.base_path, self.id, 'os-security-groups')
 
