@@ -144,7 +144,6 @@ class TestVolumeActions(TestVolume):
         self.resp.status_code = 200
         self.resp.json = mock.Mock(return_value=self.resp.body)
         self.sess = mock.Mock(spec=adapter.Adapter)
-        self.sess.default_microversion = '3.0'
         self.sess.post = mock.Mock(return_value=self.resp)
         self.sess._get_connection = mock.Mock(return_value=self.cloud)
 
@@ -153,66 +152,54 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.extend(self.sess, '20'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {"os-extend": {"new_size": "20"}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_set_volume_readonly(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_readonly(self.sess, True))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-update_readonly_flag': {'readonly': True}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_set_volume_readonly_false(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_readonly(self.sess, False))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-update_readonly_flag': {'readonly': False}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_set_volume_bootable(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_bootable_status(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_bootable': {'bootable': True}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_set_volume_bootable_false(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_bootable_status(self.sess, False))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_bootable': {'bootable': False}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_set_image_metadata(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.set_image_metadata(self.sess, {'foo': 'bar'}))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-set_image_metadata': {'foo': 'bar'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_delete_image_metadata(self):
         _volume = copy.deepcopy(VOLUME)
@@ -224,17 +211,13 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.delete_image_metadata(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body_a = {'os-unset_image_metadata': 'foo'}
         body_b = {'os-unset_image_metadata': 'baz'}
         self.sess.post.assert_has_calls(
             [
-                mock.call(
-                    url, json=body_a, microversion=sot._max_microversion
-                ),
-                mock.call(
-                    url, json=body_b, microversion=sot._max_microversion
-                ),
+                mock.call(url, json=body_a),
+                mock.call(url, json=body_b),
             ]
         )
 
@@ -243,18 +226,16 @@ class TestVolumeActions(TestVolume):
 
         self.assertIsNone(sot.delete_image_metadata_item(self.sess, 'foo'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-unset_image_metadata': 'foo'}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_reset_status(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.reset_status(self.sess, '1', '2', '3'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-reset_status': {
                 'status': '1',
@@ -262,101 +243,83 @@ class TestVolumeActions(TestVolume):
                 'migration_status': '3',
             }
         }
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_reset_status__single_option(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.reset_status(self.sess, status='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-reset_status': {
                 'status': '1',
             }
         }
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_attach_instance(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.attach(self.sess, '1', '2'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-attach': {'mountpoint': '1', 'instance_uuid': '2'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_detach(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.detach(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-detach': {'attachment_id': '1'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_detach_force(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.detach(self.sess, '1', force=True))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-force_detach': {'attachment_id': '1'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_unmanage(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.unmanage(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-unmanage': None}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_retype(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.retype(self.sess, '1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-retype': {'new_type': '1'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_retype_mp(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.retype(self.sess, '1', migration_policy='2'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-retype': {'new_type': '1', 'migration_policy': '2'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_migrate(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.migrate(self.sess, host='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-migrate_volume': {'host': '1'}}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_migrate_flags(self):
         sot = volume.Volume(**VOLUME)
@@ -367,7 +330,7 @@ class TestVolumeActions(TestVolume):
             )
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume': {
                 'host': '1',
@@ -375,22 +338,18 @@ class TestVolumeActions(TestVolume):
                 'lock_volume': True,
             }
         }
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_complete_migration(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.complete_migration(self.sess, new_volume_id='1'))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume_completion': {'new_volume': '1', 'error': False}
         }
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_complete_migration_error(self):
         sot = volume.Volume(**VOLUME)
@@ -399,21 +358,17 @@ class TestVolumeActions(TestVolume):
             sot.complete_migration(self.sess, new_volume_id='1', error=True)
         )
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {
             'os-migrate_volume_completion': {'new_volume': '1', 'error': True}
         }
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
 
     def test_force_delete(self):
         sot = volume.Volume(**VOLUME)
 
         self.assertIsNone(sot.force_delete(self.sess))
 
-        url = 'volumes/%s/action' % FAKE_ID
+        url = f'volumes/{FAKE_ID}/action'
         body = {'os-force_delete': None}
-        self.sess.post.assert_called_with(
-            url, json=body, microversion=sot._max_microversion
-        )
+        self.sess.post.assert_called_with(url, json=body)
