@@ -11,6 +11,8 @@
 # under the License.
 
 import io
+import os.path
+import tempfile
 from unittest import mock
 
 import requests
@@ -21,6 +23,7 @@ from openstack.image.v2 import cache as _cache
 from openstack.image.v2 import image as _image
 from openstack.image.v2 import member as _member
 from openstack.image.v2 import metadef_namespace as _metadef_namespace
+from openstack.image.v2 import metadef_object as _metadef_object
 from openstack.image.v2 import metadef_resource_type as _metadef_resource_type
 from openstack.image.v2 import metadef_schema as _metadef_schema
 from openstack.image.v2 import schema as _schema
@@ -46,7 +49,7 @@ class FakeResponse:
 
 class TestImageProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestImageProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
         self.proxy._connection = self.cloud
 
@@ -118,6 +121,61 @@ class TestImage(TestImageProxy):
             },
         )
 
+    def test_image_create_file_as_name(self):
+        # if we pass a filename as an image name, we should upload the file
+        # itself (and use the upload flow)
+        with tempfile.NamedTemporaryFile() as tmpfile:
+            name = os.path.basename(tmpfile.name)
+            self._verify(
+                'openstack.image.v2._proxy.Proxy._upload_image',
+                self.proxy.create_image,
+                method_kwargs={
+                    'name': tmpfile.name,
+                    'allow_duplicates': True,
+                },
+                expected_args=[
+                    name,
+                ],
+                expected_kwargs={
+                    'filename': tmpfile.name,
+                    'data': None,
+                    'meta': {},
+                    'wait': False,
+                    'timeout': 3600,
+                    'validate_checksum': False,
+                    'use_import': False,
+                    'stores': None,
+                    'all_stores': None,
+                    'all_stores_must_succeed': None,
+                    'disk_format': 'qcow2',
+                    'container_format': 'bare',
+                    'properties': {
+                        'owner_specified.openstack.md5': '',
+                        'owner_specified.openstack.object': f'images/{name}',
+                        'owner_specified.openstack.sha256': '',
+                    },
+                },
+            )
+
+        # but not if we use a directory...
+        with tempfile.TemporaryDirectory() as tmpdir:
+            self.verify_create(
+                self.proxy.create_image,
+                _image.Image,
+                method_kwargs={
+                    'name': tmpdir,
+                    'allow_duplicates': True,
+                },
+                expected_kwargs={
+                    'container_format': 'bare',
+                    'disk_format': 'qcow2',
+                    'name': tmpdir,
+                    'owner_specified.openstack.md5': '',
+                    'owner_specified.openstack.object': f'images/{tmpdir}',
+                    'owner_specified.openstack.sha256': '',
+                },
+            )
+
     def test_image_create_checksum_match(self):
         fake_image = _image.Image(
             id="fake",
@@ -271,12 +329,85 @@ class TestImage(TestImageProxy):
             data="data",
             container_format="bare",
             disk_format="raw",
-            **properties
+            **properties,
         )
 
         args, kwargs = self.proxy._create.call_args
         self.assertEqual(kwargs["is_protected"], True)
 
+    def test_image_create_with_stores(self):
+        self.proxy.find_image = mock.Mock()
+        self.proxy._upload_image = mock.Mock()
+
+        self.proxy.create_image(
+            name='fake',
+            data=b'fake',
+            container='bare',
+            disk_format='raw',
+            use_import=True,
+            stores=['cinder', 'swift'],
+        )
+
+        self.proxy.find_image.assert_called_with('fake')
+
+        self.proxy._upload_image.assert_called_with(
+            'fake',
+            container_format='bare',
+            disk_format='raw',
+            filename=None,
+            data=b'fake',
+            meta={},
+            properties={
+                self.proxy._IMAGE_MD5_KEY: '',
+                self.proxy._IMAGE_SHA256_KEY: '',  # noqa: E501
+                self.proxy._IMAGE_OBJECT_KEY: 'bare/fake',
+            },
+            timeout=3600,
+            validate_checksum=False,
+            use_import=True,
+            stores=['cinder', 'swift'],
+            all_stores=None,
+            all_stores_must_succeed=None,
+            wait=False,
+        )
+
+    def test_image_create_with_all_stores(self):
+        self.proxy.find_image = mock.Mock()
+        self.proxy._upload_image = mock.Mock()
+
+        self.proxy.create_image(
+            name='fake',
+            data=b'fake',
+            container='bare',
+            disk_format='raw',
+            use_import=True,
+            all_stores=True,
+            all_stores_must_succeed=True,
+        )
+
+        self.proxy.find_image.assert_called_with('fake')
+
+        self.proxy._upload_image.assert_called_with(
+            'fake',
+            container_format='bare',
+            disk_format='raw',
+            filename=None,
+            data=b'fake',
+            meta={},
+            properties={
+                self.proxy._IMAGE_MD5_KEY: '',
+                self.proxy._IMAGE_SHA256_KEY: '',  # noqa: E501
+                self.proxy._IMAGE_OBJECT_KEY: 'bare/fake',
+            },
+            timeout=3600,
+            validate_checksum=False,
+            use_import=True,
+            stores=None,
+            all_stores=True,
+            all_stores_must_succeed=True,
+            wait=False,
+        )
+
     def test_image_upload_no_args(self):
         # container_format and disk_format are required args
         self.assertRaises(exceptions.InvalidRequest, self.proxy.upload_image)
@@ -465,7 +596,7 @@ class TestMember(TestImageProxy):
             self.proxy.remove_member,
             method_args=["member_id"],
             method_kwargs={"image": "image_id", "ignore_missing": False},
-            expected_args=[_member.Member],
+            expected_args=[_member.Member, None],
             expected_kwargs={
                 "member_id": "member_id",
                 "image_id": "image_id",
@@ -479,7 +610,7 @@ class TestMember(TestImageProxy):
             self.proxy.remove_member,
             method_args=["member_id"],
             method_kwargs={"image": "image_id"},
-            expected_args=[_member.Member],
+            expected_args=[_member.Member, None],
             expected_kwargs={
                 "member_id": "member_id",
                 "image_id": "image_id",
@@ -492,7 +623,7 @@ class TestMember(TestImageProxy):
             "openstack.proxy.Proxy._update",
             self.proxy.update_member,
             method_args=['member_id', 'image_id'],
-            expected_args=[_member.Member],
+            expected_args=[_member.Member, None],
             expected_kwargs={'member_id': 'member_id', 'image_id': 'image_id'},
         )
 
@@ -569,6 +700,75 @@ class TestMetadefNamespace(TestImageProxy):
         )
 
 
+class TestMetadefObject(TestImageProxy):
+    def test_create_metadef_object(self):
+        self.verify_create(
+            self.proxy.create_metadef_object,
+            _metadef_object.MetadefObject,
+            method_kwargs={"namespace": "test_namespace_name"},
+            expected_kwargs={"namespace_name": "test_namespace_name"},
+        )
+
+    def test_get_metadef_object(self):
+        self.verify_get(
+            self.proxy.get_metadef_object,
+            _metadef_object.MetadefObject,
+            method_kwargs={"namespace": "test_namespace_name"},
+            expected_kwargs={
+                "namespace_name": "test_namespace_name",
+                'name': 'resource_id',
+            },
+            expected_args=[],
+        )
+
+    def test_metadef_objects(self):
+        self.verify_list(
+            self.proxy.metadef_objects,
+            _metadef_object.MetadefObject,
+            method_kwargs={"namespace": "test_namespace_name"},
+            expected_kwargs={"namespace_name": "test_namespace_name"},
+        )
+
+    def test_update_metadef_object(self):
+        self._verify(
+            "openstack.proxy.Proxy._update",
+            self.proxy.update_metadef_object,
+            method_args=["test_metadef_object", "test_namespace_name"],
+            method_kwargs={"name": "new_object"},
+            expected_args=[
+                _metadef_object.MetadefObject,
+                'test_metadef_object',
+            ],
+            expected_kwargs={
+                "name": "new_object",
+                "namespace_name": "test_namespace_name",
+            },
+        )
+
+    def test_delete_metadef_object(self):
+        self.verify_delete(
+            self.proxy.delete_metadef_object,
+            _metadef_object.MetadefObject,
+            False,
+            method_kwargs={"namespace": "test_namespace_name"},
+            expected_kwargs={"namespace_name": "test_namespace_name"},
+        )
+
+    @mock.patch.object(proxy_base.Proxy, '_get_resource')
+    def test_delete_all_metadef_objects(self, mock_get_resource):
+        fake_object = _metadef_namespace.MetadefNamespace()
+        mock_get_resource.return_value = fake_object
+        self._verify(
+            "openstack.image.v2.metadef_namespace.MetadefNamespace.delete_all_objects",
+            self.proxy.delete_all_metadef_objects,
+            method_args=['test_namespace'],
+            expected_args=[self.proxy],
+        )
+        mock_get_resource.assert_called_once_with(
+            _metadef_namespace.MetadefNamespace, 'test_namespace'
+        )
+
+
 class TestMetadefResourceType(TestImageProxy):
     def test_metadef_resource_types(self):
         self.verify_list(
@@ -926,3 +1126,12 @@ class TestCache(TestImageProxy):
             expected_args=[self.proxy, 'both'],
         )
         mock_get_resource.assert_called_once_with(_cache.Cache, None)
+
+        mock_get_resource.reset_mock()
+        self._verify(
+            "openstack.image.v2.cache.Cache.clear",
+            self.proxy.clear_cache,
+            method_args=[],
+            expected_args=[self.proxy, 'both'],
+        )
+        mock_get_resource.assert_called_once_with(_cache.Cache, None)
